# -*- coding: utf-8 -*-

# Authors: Alejandro J. Cura <alecu@canonical.com>
#          Roberto Alsina <roberto.alsina@canonical.com>
#          Diego Sarmentero <diego.sarmentero@canonical.com>
#
# Copyright 2011 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""The user interface for the Ubuntu One Installer."""

import gettext

from PyQt4 import QtGui, QtCore

from ubuntu_sso.qt.gui import (
    SSOWizardPage,
    EmailVerificationPage,
    ErrorPage,
    ForgottenPasswordPage,
    ResetPasswordPage,
)
from ubuntu_sso.qt.current_user_sign_in_ui import \
    Ui_CurrentUserSignInPage
from ubuntu_sso.qt.email_verification_ui import \
    Ui_EmailVerificationPage
from ubuntu_sso.qt.error_message_ui import Ui_ErrorPage
from ubuntu_sso.qt.success_message_ui import Ui_SuccessPage
from ubuntu_sso.qt.forgotten_password_ui import \
    Ui_ForgottenPasswordPage
from ubuntu_sso.qt.reset_password_ui import Ui_ResetPasswordPage
from ubuntu_sso.qt.controllers import (
    ChooseSignInController,
    EmailVerificationController,
    ErrorController,
    ResetPasswordController,
    SetUpAccountController,
    SuccessController,
)

from ubuntuone.controlpanel.gui.qt.loadingoverlay import LoadingOverlay

from ubuntuone.platform.credentials import (
    APP_NAME,
    TC_URL,
)

from ubuntuone_installer.logger import setup_logging
from ubuntuone_installer.gui.qt.ui import (
    choose_sign_in_ui,
    congratulations_ui,
    license_ui,
    setup_account_ui,
)

from ubuntuone_installer.gui import qt
from ubuntuone_installer.gui.qt.are_you_sure import AreYouSure
from ubuntuone_installer.gui.qt.currentuser import CurrentUserController
from ubuntuone_installer.gui.qt.folders import FoldersPage
from ubuntuone_installer.gui.qt.forgotten import ForgottenPasswordController
from ubuntuone_installer.gui.qt.local_folders import LocalFoldersPage
from ubuntuone_installer.gui.qt.preferences import PreferencesPage
from ubuntuone_installer.gui.qt.sync_now_or_later import SyncNowOrLaterPage
from ubuntuone_installer.gui.qt.setup_account import SetupAccountPage
from ubuntuone_installer.gui.qt.side_widget import SideWidget

_ = gettext.gettext
SIGN_IN = _("Sign in to Ubuntu One")
SIGN_IN_SUBTITLE = _("Sign in with your existing Ubuntu One"
    " username and password.")
TITLE_STYLE = "<span style=\"font-size:24px\">%s</span>"
CREDENTIALS_ERROR_TITLE = _("Error getting credentials")
CREDENTIALS_ERROR = _('Application will close.\n\n%r')
SKIP_TOUR = _("Skip tour, go to Dashboard")
START_SETUP = _("Start setup")
UPDATE_TITLE = _("Updates are available")
UPDATE_SOFTWARE = _("There is a new version available. "
                    "Do you want perform an upgrade?")
# Invalid name logger
# pylint: disable=C0103
logger = setup_logging('qt.gui')
# pylint: enable=C0103


def user_wants_to_update():
    """Ask the user if he really wants to update the software."""
    result = QtGui.QMessageBox.question(None, UPDATE_TITLE, UPDATE_SOFTWARE,
            QtGui.QMessageBox.Yes, QtGui.QMessageBox.No)
    if result == QtGui.QMessageBox.Yes:
        return True
    return False


class LicensePage(SSOWizardPage):
    """Wizard Page that displays the license info and links to the GPL."""

    def __init__(self, parent=None):
        self._next_id = None
        super(LicensePage, self).__init__(license_ui.Ui_Form(), None, parent)
        self.header.setVisible(False)
        self.ui.textBrowser.setHtml(qt.LICENSE_CONTENT)

    # Invalid names of Qt-inherited methods
    # pylint: disable=C0103

    def initializePage(self):
        """Setup UI details."""
        # Set the right texts and connections for buttons
        self.setButtonText(QtGui.QWizard.NextButton, _("Agree && continue"))
        self.setButtonText(QtGui.QWizard.CancelButton,
            _("Disagree && uninstall"))

        # This is just to catch an exception thrown when nothing
        # is connected to the signal. It's not an exceptional
        # condition at all.
        try:
            self.wizard().customButtonClicked.disconnect()
        except TypeError:
            pass

        self.wizard().setButtonLayout([
            QtGui.QWizard.CancelButton,
            QtGui.QWizard.BackButton,
            QtGui.QWizard.Stretch,
            QtGui.QWizard.NextButton,
            QtGui.QWizard.FinishButton])

    def nextId(self):
        """Return next page's ID."""
        if self._next_id is None:
            return self.wizard().SIGNIN_PAGE
        return self._next_id


class SignInPage(SSOWizardPage):
    """Wizard Page that lets the user Sign into Ubuntu One."""

    # Invalid names of Qt-inherited methods
    # pylint: disable=C0103

    def initializePage(self):
        """Setup UI details."""
        self.ui.cancel_button.setText(_("Close window and setup later"))
        self.ui.cancel_button.clicked.connect(self.wizard().reject)
        # Layout without custom button 1,
        # without finish button
        # without cancel
        self.wizard().setButtonLayout([
            QtGui.QWizard.Stretch,
            QtGui.QWizard.NextButton])

        # Set existing_account_button as default when the page is shown.
        self.ui.existing_account_button.setDefault(True)
        self.ui.existing_account_button.style().unpolish(
            self.ui.existing_account_button)
        self.ui.existing_account_button.style().polish(
            self.ui.existing_account_button)

    def nextId(self):
        """Provide the next id."""
        return self.next


class CurrentUserSignInPage(SSOWizardPage):
    """Wizard Page that lets a current user Sign into Ubuntu One."""

    # Invalid names of Qt-inherited methods
    # pylint: disable=C0103

    def nextId(self):
        """Provide the next id."""
        return self.next

    def initializePage(self):
        """Setup UI details."""
        self.setButtonText(QtGui.QWizard.CancelButton,
            _("Cancel"))
        # Layout without custom button 1,
        # without finish button
        self.wizard().setButtonLayout([
            QtGui.QWizard.BackButton,
            QtGui.QWizard.Stretch])

        # Set sign_in_button as default when the page is shown.
        self.ui.sign_in_button.setDefault(True)
        self.ui.sign_in_button.setEnabled(False)
        self.ui.sign_in_button.setProperty("DisabledState",
            not self.ui.sign_in_button.isEnabled())
        self.ui.sign_in_button.style().unpolish(
            self.ui.sign_in_button)
        self.ui.sign_in_button.style().polish(
            self.ui.sign_in_button)


class SuccessPage(SSOWizardPage):
    """Shown after SSO login, before setup."""

    # Invalid names of Qt-inherited methods
    # pylint: disable=C0103

    def __init__(self, parent=None):
        super(SuccessPage, self).__init__(
            Ui_SuccessPage(), SuccessController(), parent)

    def initializePage(self):
        """Setup UI details."""
        self.wizard().setButtonLayout([
            QtGui.QWizard.Stretch,
            QtGui.QWizard.FinishButton,
            QtGui.QWizard.NextButton,
            ])
        self.setFinalPage(True)
        self.wizard().setButtonText(QtGui.QWizard.FinishButton,
            SKIP_TOUR)
        self.wizard().setButtonText(QtGui.QWizard.NextButton,
            START_SETUP)

    def nextId(self):
        """Provide the next id."""
        return self.next


class CongratulationsPage(SSOWizardPage):
    """Final page of the wizard."""

    def __init__(self, parent=None):
        super(CongratulationsPage, self).__init__(
            congratulations_ui.Ui_Form(), None, parent)
        self.setFinalPage(True)

    # Invalid names of Qt-inherited methods
    # pylint: disable=C0103

    def initializePage(self):
        """Setup UI details."""
        # We need custom buttons
        self.wizard().setButtonText(QtGui.QWizard.FinishButton,
            _("Go to my Ubuntu One dashboard"))
        # Layout without custom button 1,
        self.wizard().setButtonLayout([
            QtGui.QWizard.Stretch,
            QtGui.QWizard.FinishButton])
        # Set FinishButton as default when the page is shown.
        self.wizard().button(QtGui.QWizard.FinishButton).setDefault(True)
        self.wizard().button(QtGui.QWizard.FinishButton).style().unpolish(
            self.wizard().button(QtGui.QWizard.FinishButton))
        self.wizard().button(QtGui.QWizard.FinishButton).style().polish(
            self.wizard().button(QtGui.QWizard.FinishButton))

        # Add syncdaemon to autostart
        qt.utils.add_syncdaemon_to_autostart()


class MainWindow(QtGui.QWizard):

    """The Main Window of the Installer wizard."""

    # Invalid constant names and Qt-inherited methods
    # pylint: disable=C0103

    LICENSE_PAGE = 0
    SIGNIN_PAGE = 1
    SYNC_NOW_OR_LATER_PAGE = 2
    CONGRATULATIONS_PAGE = 3

    # definition of the signals raised by the widget (for SSO)
    recoverableError = QtCore.pyqtSignal('QString', 'QString')
    loginSuccess = QtCore.pyqtSignal('QString', 'QString')
    registrationSuccess = QtCore.pyqtSignal('QString', 'QString')
    userCancellation = QtCore.pyqtSignal('QString')

    def __init__(self, close_callback=None, installing=False):
        """Initialize this instance."""
        # Used to decide the next page dynamically
        self._next_id = None

        # Data for SSO
        self.app_name = APP_NAME
        self.tc_url = TC_URL
        # help_text is not displayed in the UI from design
        self.help_text = ""

        super(MainWindow, self).__init__()
        # Set windows flags
        flags = self.windowFlags() & ~QtCore.Qt.WindowContextHelpButtonHint
        self.setWindowFlags(flags)

        self.setWindowTitle(APP_NAME)
        self.setMinimumHeight(538)
        self.setMinimumWidth(740)
        self.setWizardStyle(self.ModernStyle)
        self.close_callback = close_callback

        # Label for Form Errors
        self.form_errors_label = QtGui.QLabel()
        self.form_errors_label.setObjectName("form_errors")
        self.form_errors_label.hide()

        self.setSideWidget(SideWidget())
        self.overlay = LoadingOverlay(self)
        self.overlay.hide()

        self.setOption(self.NoBackButtonOnStartPage, True)

        # PyQt doesn't support the (int, page) version of addPage, so
        # Add the pages in the right order
        # pylint: disable=C0103

        self.license_page = LicensePage()
        self.LICENSE_PAGE = self.addPage(self.license_page)

        #SSO Pages
        title_page = TITLE_STYLE % SIGN_IN
        self.sign_in_controller = ChooseSignInController(
            title=title_page)
        self.sign_in_page = SignInPage(
            ui=choose_sign_in_ui.Ui_ChooseSignInPage(),
            controller=self.sign_in_controller,
            parent=self)
        if not installing:
            self.sign_in_page.ui.message_label.setText("")

        self.setup_controller = SetUpAccountController(message_box=self)
        self.setup_account = SetupAccountPage(
            setup_account_ui.Ui_SetUpAccountPage(),
            self.setup_controller,
            parent=self)

        email_controller = EmailVerificationController(message_box=self)
        email_verification_page = Ui_EmailVerificationPage()
        self.email_verification = EmailVerificationPage(
                                                       email_verification_page,
                                                       email_controller)
        self.email_verification.next_button.clicked.connect(
            self.overlay.show)

        title_page = TITLE_STYLE % SIGN_IN
        self.current_user_controller = CurrentUserController(
            title=title_page, subtitle=SIGN_IN_SUBTITLE,
            message_box=self)
        self.current_user = CurrentUserSignInPage(Ui_CurrentUserSignInPage(),
                                                  self.current_user_controller,
                                                  parent=self)
        self.current_user.ui.sign_in_button.clicked.connect(
            self.overlay.show)

        self.success = SuccessPage(self)
        self.error_controller = ErrorController()
        self.error = ErrorPage(Ui_ErrorPage(), self.error_controller)

        self.forgotten_pwd_controller = ForgottenPasswordController(
            message_box=self)
        self.forgotten = ForgottenPasswordPage(Ui_ForgottenPasswordPage(),
                                               self.forgotten_pwd_controller,
                                               parent=self)
        self.forgotten.ui.send_button.clicked.connect(
            self.overlay.show)

        self.reset_password_controller = ResetPasswordController(
            message_box=self)
        self.reset_password = ResetPasswordPage(Ui_ResetPasswordPage(),
                                                self.reset_password_controller,
                                                parent=self)
        self.reset_password.ui.reset_password_button.clicked.connect(
            self.overlay.show)

        self.SIGNIN_PAGE = self.addPage(self.sign_in_page)
        self.setup_account_page_id = self.addPage(self.setup_account)
        self.email_verification_page_id = self.addPage(self.email_verification)
        self.current_user_page_id = self.addPage(self.current_user)
        self.SUCCESS_PAGE = self.addPage(self.success)
        self.ERROR_PAGE = self.addPage(self.error)
        self.forgotten_password_page_id = self.addPage(self.forgotten)
        self.reset_password_page_id = self.addPage(self.reset_password)

        self.loginSuccess.connect(self.login_success_slot)
        self.registrationSuccess.connect(self.registration_success_slot)

        # End of SSO pages

        self.SYNC_NOW_OR_LATER_PAGE = self.addPage(SyncNowOrLaterPage())
        self.folders_page = FoldersPage()
        self.folders_page_id = self.addPage(self.folders_page)
        self.local_folders_page = LocalFoldersPage()
        self.local_folders_page_id = self.addPage(self.local_folders_page)
        self.preferences_page = PreferencesPage()
        self.preferences_page_id = self.addPage(self.preferences_page)
        self.CONGRATULATIONS_PAGE = self.addPage(CongratulationsPage())

        self.currentIdChanged.connect(self.on_currentIdChanged)
        # Set Wizard buttons style
        self.button(QtGui.QWizard.NextButton).setDefault(True)

        if installing:
            self.setStartId(self.LICENSE_PAGE)
        else:
            self.setStartId(self.SIGNIN_PAGE)
        self.restart()

    # Invalid name "closeEvent"
    # pylint: disable=C0103

    def critical(self, message, page=None):
        """Show a message at the bottom of the page on form errors."""
        self.overlay.hide()
        self.form_errors_label.setText(message)
        self.form_errors_label.show()
        if page is None and self.currentPage():
            self.currentPage().layout().addWidget(self.form_errors_label)
        elif page is not None:
            page.layout().addWidget(self.form_errors_label)

    def next(self):
        """Show the next page to display and remove the form errors label."""
        self.overlay.hide()
        self.form_errors_label.hide()
        if self.currentPage().layout():
            self.currentPage().layout().removeWidget(self.form_errors_label)
        self.form_errors_label.setParent(None)
        super(MainWindow, self).next()

    def nextId(self):
        """Selects which page to display next."""
        if self._next_id is not None:
            return self._next_id
        else:
            return QtGui.QWizard.nextId(self)

    def login_success_slot(self):
        """Called on successful login."""
        self._next_id = self.SUCCESS_PAGE
        self.next()
        self._next_id = self.SYNC_NOW_OR_LATER_PAGE

    def registration_success_slot(self):
        """Called on successful registration."""
        self._next_id = self.SUCCESS_PAGE
        self.next()
        self._next_id = self.SYNC_NOW_OR_LATER_PAGE

    def done(self, result):
        """The main window is being closed, call any custom callback."""
        # Finished, not cancelled
        if result == 1:  # Cancelled
            qt.utils.start_control_panel()
        elif self.currentId() == self.LICENSE_PAGE:
            if not AreYouSure(self).exec_():
                qt.utils.uninstall_application()
            else:
                return
        elif self.currentId() != self.CONGRATULATIONS_PAGE:
            if not AreYouSure(self).exec_():
                return
        if self.close_callback is not None:
            self.close_callback()
        QtGui.QWizard.done(self, result)
        self.userCancellation.emit(self.app_name)

    def on_currentIdChanged(self, page_id):
        """The current page changed."""
        if page_id < self.SIGNIN_PAGE:
            self.sideWidget().ui.states_frame.hide()
            self.sideWidget().set_stage(0)
        elif page_id < self.SYNC_NOW_OR_LATER_PAGE:
            self.sideWidget().ui.states_frame.show()
            self.sideWidget().set_stage(1)
        elif page_id < self.CONGRATULATIONS_PAGE:
            self.sideWidget().set_stage(3)
        else:
            self.sideWidget().set_stage(4)

    # Invalid name "resizeEvent"
    # pylint: disable=C0103

    def resizeEvent(self, event):
        """Resize the overlay to fit all the widget."""
        QtGui.QWizard.resizeEvent(self, event)
        self.overlay.resize(event.size())
