# -*- coding: utf-8 -*-

# Authors: Alejandro J. Cura <alecu@canonical.com>
#          Roberto Alsina <roberto.alsina@canonical.com>
#
# Copyright 2011 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Tests for the Windows main."""

from twisted.internet import defer

from ubuntuone_installer.gui.qt import gui, utils
from ubuntuone_installer.gui.qt.main import windows
from ubuntuone_installer.gui.qt.tests.test_gui import FakeMainWindow
from ubuntuone_installer.tests import TestCase


class FakeFailingCredentialsManagementTool(object):

    """A fake CredentialsManagementTool that fails."""

    def find_credentials(self):
        """Return a failure."""
        return defer.fail(Exception())


class FakeNoCredentialsManagementTool(object):

    """A fake CredentialsManagementTool that fakes no credentials."""

    def find_credentials(self):
        """Return a success, but no credentials."""
        return defer.succeed({})


class FakeCredentialsManagementTool(object):

    """A fake CredentialsManagementTool that fakes credentials."""

    def find_credentials(self):
        """Return a success, but no credentials."""
        return defer.succeed("Something not false")


class FakeLogger(object):

    """Fake Logger."""

    error_args = None
    debug_args = None
    info_args = None

    def error(self, *args, **kwargs):
        """Save the error."""
        self.error_args = (args, kwargs)

    def debug(self, *args, **kwargs):
        """Save the debug."""
        self.debug_args = (args, kwargs)

    def info(self, *args, **kwargs):
        """Save the info."""
        self.info_args = (args, kwargs)


class CredsFailureMainTestCase(TestCase):

    """Test the qt main window."""

    stopped = False
    updates_checked = False

    def stop(self):
        """A fake stop."""
        self.stopped = True

    def check_updates(self, gui_ref, logger):
        """A fake check updates."""
        self.updates_checked = True

    @defer.inlineCallbacks
    def test_failure(self):
        """Credential errors should display a critical message."""
        logger = FakeLogger()
        self.patch(windows, 'stop', self.stop)
        self.stopped = False
        yield windows.check_credentials(
            FakeFailingCredentialsManagementTool,
            gui, logger)
        # Should log the error
        self.assertEqual(logger.error_args[0][0],
            ('Error while getting the credentials: %r'))
        # Should stop the app
        self.assertTrue(self.stopped)

    @defer.inlineCallbacks
    def test_no_creds(self):
        """No credentials should start the wizard."""
        logger = FakeLogger()
        fakemain = FakeMainWindow()
        self.patch(gui, 'MainWindow', lambda *args, **kwargs: fakemain)
        self.patch(windows, 'stop', self._set_called)
        self.assertFalse(fakemain.shown)
        self.patch(windows, 'stop', self.stop)
        self.stopped = False
        yield windows.check_credentials(
            FakeNoCredentialsManagementTool,
            gui, logger)
        # Should show the main window
        self.assertTrue(fakemain.shown)
        # Should not log errors
        self.assertEqual(logger.error_args, None)
        # Should not stop the app
        self.assertFalse(self.stopped)

    @defer.inlineCallbacks
    def test_with_creds(self):
        """Having credentials should start the control panel."""
        logger = FakeLogger()
        self.patch(utils, 'start_control_panel', self._set_called)
        self.patch(windows, 'stop', self.stop)
        self.patch(windows, 'check_updates', self.check_updates)
        self.stopped = False
        yield windows.check_credentials(
            FakeCredentialsManagementTool,
            gui, logger)
        # Should not log errors
        self.assertEqual(logger.error_args, None)
        # Should start u1cp
        self.assertEqual(self._called, ((), {'with_icon': False}))
        # Should stop the app
        self.assertTrue(self.stopped)
        self.assertTrue(self.updates_checked)

    @defer.inlineCallbacks
    def test_without_installed_flag(self):
        """Test behaviour without the installer flag."""
        logger = FakeLogger()
        self.patch(windows, "success_cb", self._set_called)
        yield windows.check_credentials(
            FakeCredentialsManagementTool,
            gui, logger)
        self.assertEqual(self._called, (('Something not false',
            gui, False, logger), {}))

    @defer.inlineCallbacks
    def test_with_installed_flag(self):
        """Test behaviour without the installer flag."""
        logger = FakeLogger()
        self.patch(windows, "success_cb", self._set_called)
        yield windows.check_credentials(
            FakeCredentialsManagementTool,
            gui, logger, installer=True)
        self.assertEqual(self._called, (('Something not false',
            gui, True, logger), {}))
