# -*- coding: utf-8 -*-

# Authors: Roberto Alsina <roberto.alsina@canonical.com>
#
# Copyright 2011 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Customized Setup Account page for SSO."""

import gettext
import re

from PyQt4 import QtGui, QtCore

from ubuntu_sso.qt import common
from ubuntu_sso.qt import gui as sso_gui
from ubuntu_sso.utils.ui import SET_UP_ACCOUNT_BUTTON

from ubuntuone_installer.gui.qt import enhanced_check_box

_ = gettext.gettext

# pylint: disable=C0103
ERROR = u'<font color="#df2d1f"><b> %s </b></font>'
TITLE_STYLE = "<span style=\"font-size:24px\">%s</span>"

EMAIL = _("Email")
EMPTY_NAME = _("Please enter your name")
INVALID_EMAIL = _("Please enter a valid email address")
EMAIL_MATCH = _("The email addresses do not match")
NAME = _("Name")
PASSWORD = _("Create a password")
RETYPE_EMAIL = _("Retype email")
RETYPE_PASSWORD = _("Retype password")
SUBTITLE = _("You only need to set up your account "
             "once to get access to Ubuntu One across your devices.")
TERMS = _("By signing up to Ubuntu One you agree to our "
          "{terms_and_conditions} and {privacy_policy}")
TERMS_LINK = _("<a href='https://one.ubuntu.com/terms/'>"
               "<span style='color:#df2d1f;'>Terms of Service</span></a>")
PRIVACY_POLICY_LINK = _("<a href='https://one.ubuntu.com/privacy/'>"
                        "<span style='color:#df2d1f;'>Privacy Policy"
                        "</span></a></font>")
TITLE = _("Sign Up to Ubuntu One")


class SetupAccountPage(sso_gui.SetupAccountPage):
    """Customized Setup Account page for SSO."""

    def __init__(self, *args, **kwargs):
        super(SetupAccountPage, self).__init__(*args, **kwargs)
        self.ui.password_edit.textEdited.connect(
            lambda: common.password_assistance(self.ui.password_edit,
                                                 self.ui.password_assistance,
                                                 common.NORMAL))
        terms = TERMS.format(terms_and_conditions=TERMS_LINK,
                         privacy_policy=PRIVACY_POLICY_LINK)
        self.terms_checkbox = enhanced_check_box.EnhancedCheckBox(terms)
        self.ui.hlayout_check.addWidget(self.terms_checkbox)

        self.set_up_button = None
        self.captcha_received = False

    # Invalid name "initializePage"
    # pylint: disable=C0103

    def initializePage(self):
        """Setup UI details."""
        # We need to override some texts from SSO
        # to match our spec
        title_page = TITLE_STYLE % TITLE
        self.setTitle(title_page)
        self.setSubTitle(SUBTITLE)
        # Set Setup Account button
        self.wizard().setOption(QtGui.QWizard.HaveCustomButton3, True)
        try:
            self.wizard().customButtonClicked.disconnect()
        except TypeError:
            pass
        self.setButtonText(QtGui.QWizard.CustomButton3, SET_UP_ACCOUNT_BUTTON)
        self.set_up_button = self.wizard().button(QtGui.QWizard.CustomButton3)
        self.set_up_button.clicked.connect(self.wizard().overlay.show)
        self.set_up_button.clicked.connect(self.controller.set_next_validation)
        self.set_up_button.setEnabled(False)

        self.ui.name_label.setText(NAME)
        self.ui.email_label.setText(EMAIL)
        self.ui.confirm_email_label.setText(RETYPE_EMAIL)
        self.ui.password_label.setText(PASSWORD)
        self.ui.confirm_password_label.setText(RETYPE_PASSWORD)
        self.ui.password_info_label.hide()

        # Button setup
        self.wizard().setButtonLayout([
            QtGui.QWizard.BackButton,
            QtGui.QWizard.Stretch,
            QtGui.QWizard.CustomButton3])

        common.password_default_assistance(self.ui.password_assistance)
        # Hide assistance labels by default
        self.ui.name_assistance.setVisible(False)
        self.ui.email_assistance.setVisible(False)
        self.ui.confirm_email_assistance.setVisible(False)
        self.ui.password_assistance.setVisible(False)
        self.ui.refresh_label.setVisible(True)

    def focus_changed(self, old, now):
        """Check who has the focus to activate password popups if necessary."""
        if old == self.ui.name_edit:
            self.name_assistance()
        elif old == self.ui.email_edit:
            self.email_assistance()
        elif old == self.ui.confirm_email_edit:
            self.confirm_email_assistance()
        elif old == self.ui.confirm_password_edit:
            common.password_check_match(self.ui.password_edit,
                                      self.ui.confirm_password_edit,
                                      self.ui.password_assistance)
        if now == self.ui.password_edit:
            self.ui.password_assistance.setVisible(True)

    def name_assistance(self):
        """Show help for the name field."""
        text = unicode(self.ui.name_edit.text())
        if not text.strip():
            self.set_error_message(self.ui.name_assistance,
                EMPTY_NAME)
            common.check_as_invalid(self.ui.name_edit)
        else:
            self.ui.name_assistance.setVisible(False)
            common.check_as_valid(self.ui.name_edit)

    def email_assistance(self):
        """Show help for the email field."""
        text = unicode(self.ui.email_edit.text())
        if not is_correct_email(text):
            self.set_error_message(self.ui.email_assistance,
                INVALID_EMAIL)
            common.check_as_invalid(self.ui.email_edit)
        else:
            self.ui.email_assistance.setVisible(False)
            common.check_as_valid(self.ui.email_edit)

    def confirm_email_assistance(self):
        """Show help for the confirm email field."""
        text1 = unicode(self.ui.email_edit.text())
        text2 = unicode(self.ui.confirm_email_edit.text())
        if text1 != text2:
            self.set_error_message(self.ui.confirm_email_assistance,
                EMAIL_MATCH)
            common.check_as_invalid(self.ui.confirm_email_edit)
        else:
            self.ui.confirm_email_assistance.setVisible(False)
            common.check_as_valid(self.ui.confirm_email_edit)

    def set_error_message(self, label, msg):
        """Set the message to the proper label applying the error style."""
        label.setText(ERROR % msg)
        label.setVisible(True)

    def showEvent(self, event):
        """Set set_up_button as default button when the page is shown."""
        # This method should stays here because if we move it to initializePage
        # set_up_button won't take the proper style for hover and press
        if self.set_up_button is not None:
            self.set_up_button.setVisible(True)
            self.set_up_button.setDefault(True)
            if not self.set_up_button.isEnabled():
                self.set_up_button.setProperty("DisabledState", True)
                self.set_up_button.style().unpolish(self.set_up_button)
                self.set_up_button.style().polish(self.set_up_button)
        self.connect(QtGui.QApplication.instance(),
            QtCore.SIGNAL("focusChanged(QWidget*, QWidget*)"),
            self.focus_changed)
        super(SetupAccountPage, self).showEvent(event)
        if not self.captcha_received:
            self.wizard().overlay.show()

    def hideEvent(self, event):
        """Disconnect the focusChanged signal when the page change."""
        if self.set_up_button is not None:
            self.set_up_button.setVisible(False)
        try:
            self.disconnect(QtGui.QApplication.instance(),
                QtCore.SIGNAL("focusChanged(QWidget*, QWidget*)"),
                self.focus_changed)
        except TypeError:
            pass
        super(SetupAccountPage, self).hideEvent(event)

    def on_captcha_refreshing(self):
        """Show overlay when captcha is refreshing."""
        if self.isVisible():
            self.wizard().overlay.show()
        self.captcha_received = False

    def on_captcha_refresh_complete(self):
        """Hide overlay when captcha finished refreshing."""
        self.wizard().overlay.hide()
        self.captcha_received = True


def is_min_required_password(password):
    """Return if the password meets the minimum requirements."""
    if (len(password) < 8 or
        re.search('[A-Z]', password) is None or
        re.search('\d+', password) is None):
        return False
    return True


# FIXME: this should do the same check as SSO's server side
def is_correct_email(email_address):
    """Return if the email is correct."""
    return '@' in email_address
