# -*- coding: utf-8 -*-

# Authors: Alejandro J. Cura <alecu@canonical.com>
#
# Copyright 2011 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""The test suite for the Qt UI of the Ubuntu One Installer."""

from PyQt4 import QtGui, QtCore
from ubuntuone.controlpanel.gui.qt import loadingoverlay

from ubuntuone_installer.gui.tests import FakedObject
from ubuntuone_installer.tests import TestCase

# Attribute 'yyy' defined outside __init__, access to a protected member
# pylint: disable=W0201, W0212


NO_OP = lambda *args: None


class FakeUi(FakedObject):
    """A fake Ui object."""

    exposed_methods = ['setupUi']


class FakeSignal(object):

    """A fake PyQt signal."""

    def __init__(self, *args, **kwargs):
        """Initialize."""
        self.target = None

    def connect(self, target):
        """Fake connect."""
        self.target = target

    def disconnect(self, *args):
        """Fake disconnect."""
        self.target = None

    def emit(self, *args):
        """Fake emit."""
        if self.target:
            self.target(*args)


class FakeController(object):

    """A fake controller for the tests."""

    is_fake = True
    set_next_validation = lambda *a, **kw: None

    def __init__(self, *args, **kwargs):
        self.args = (args, kwargs)

    # pylint: disable=C0103
    def setupUi(self, view):
        """Fake the setup."""
    # pylint: enable=C0103


class FakeOverlay(object):

    """A fake delay overlay."""

    def __init__(self, *args, **kwargs):
        """Initialize."""
        self.show_counter = 0
        self.hide_counter = 0
        self.args = (args, kwargs)

    def show(self):
        """Fake show."""
        self.show_counter += 1

    def hide(self):
        """Fake hide."""
        self.hide_counter += 1

    def resize(self, *args):
        """Fake resize."""


class FakeWizard(object):
    """Replace wizard() function on wizard pages."""

    customButtonClicked = QtCore.QObject()

    def __init__(self):
        self.overlay = FakeOverlay()
        self.called = []
        self.buttons = {}

    # Invalid name "setButtonLayout", "setOption"
    # pylint: disable=C0103

    def setButtonLayout(self, *args, **kwargs):
        """Fake the functionality of setButtonLayout on QWizard class."""
        self.called.append(('setButtonLayout', (args, kwargs)))

    def setOption(self, *args, **kwargs):
        """Fake the functionality of setOption on QWizard class."""
        self.called.append(('setOption', (args, kwargs)))

    # pylint: enable=C0103

    def button(self, button_id):
        """Fake the functionality of button on QWizard class."""
        return self.buttons.setdefault(button_id, QtGui.QPushButton())


class FakeWizardPage(object):

    """A fake wizard page."""

    def __init__(self, *args, **kwargs):
        self.has_back_button = True

    # pylint: disable=C0103
    def initializePage(self):
        """Fake initializePage."""


class BaseTestCase(TestCase):
    """Base Test Case."""

    class_ui = None
    kwargs = {}

    def setUp(self):
        super(BaseTestCase, self).setUp()
        # self.class_ui is not callable
        # pylint: disable=E1102
        # pylint: disable=C0103
        self.ui = None
        if self.class_ui is not None:
            self.ui = self.class_ui(**self.kwargs)

        if hasattr(self.ui, 'backend'):
            # clean backend calls
            self.ui.backend._called.clear()

        self.patch(loadingoverlay, 'LoadingOverlay', FakeOverlay)
