# -*- coding: utf-8 -*-

# Authors: Roberto Alsina <roberto.alsina@canonical.com>
#
# Copyright 2011 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Tests for the local folders page."""

import os
import Queue
import shutil
import tempfile

from PyQt4 import QtGui, QtCore
from twisted.internet import defer

from ubuntuone_installer.gui import NEXT
from ubuntuone_installer.gui.qt import local_folders
from ubuntuone_installer.gui.qt.tests import BaseTestCase, TestCase
from ubuntuone_installer.gui.qt.tests.test_gui import FakeMainWindow


class FakeFileDialog(object):

    """A fake QFileDialog class."""

    DontUseNativeDialog = 4

    # pylint: disable=C0103
    def getExistingDirectory(self, *args, **kwargs):
        """Fake existing folder name."""
        return QtCore.QString("c:\\whatever")


class FakeMessageBox(object):

    """A fake QMessageBox class."""

    Close = 2
    _warning = None

    def warning(self, *args, **kwargs):
        """Fake warning."""
        self._warning = (args, kwargs)


class FakeCPBackend(object):
    """Fake Control Panel backend."""

    def __init__(self):
        """Initialize."""
        self._is_valid = True
        self.volume_setings_changes = []
        self.folders_created = []

    def account_info(self, *args):
        """Fake account info."""
        return defer.succeed({
            "quota_total": 1000,
            "quota_used": 200,
            })

    def volumes_info(self, *args):
        """Fake volumes info."""
        return defer.succeed(((None, None,
            [
            {
                'type': u'UDF',
                'path': os.path.expanduser(u'~/xyzzy'),
                'volume_id': 'asdfgh',
                'subscribed': True,
            },
            {
                'type': u'UDF',
                'path': os.path.expanduser(u'~/zxyzzy'),
                'volume_id': 'qwerty',
                'subscribed': False,
            },
            {
                'type': u'SHARE',
                'path': os.path.expanduser(u'~/foobar'),
                'volume_id': 'shared',
                'subscribed': False,
            },
            ],
        ),))

    def validate_path_for_folder(self, path):
        """Fake folder validation."""
        return self._is_valid

    def change_volume_settings(self, *args):
        """Fake change volume settings."""
        self.volume_setings_changes.append(args)

    def create_folder(self, *args):
        """Fake folder creation."""
        self.folders_created.append(args)


class FakeFailingCPBackend(object):
    """Fake Control Panel backend that fails."""

    def account_info(self, *args):
        """Fake account info."""
        return defer.fail(Exception())

    def volumes_info(self, *args):
        """Fake account info."""
        return defer.fail(Exception())


class FakeCalculateSize(object):

    """A fake CalculateSize."""

    def __init__(self, *args, **kwargs):
        self.started = False

    def start(self):
        """Fake start."""
        self.started = True


class FolderItemTestCase(TestCase):

    """Tests for the FolderItem class."""

    def test_folder_item_new_udf(self):
        """Test when created like a new UDF."""
        self.patch(local_folders, "CalculateSize", FakeCalculateSize)
        item = local_folders.FolderItem(["s1", "s2"], "path")
        self.assertTrue(item.size is None)
        self.assertEqual(item.path, "path")
        # pylint: disable=E1101
        self.assertTrue(item.thread.started)
        self.assertEqual(item.checkState(0), QtCore.Qt.Unchecked)
        self.assertTrue(item.volume_id is None)

    def test_folder_item_existing_udf(self):
        """Test when created like an existing UDF."""
        self.patch(local_folders, "CalculateSize", FakeCalculateSize)
        item = local_folders.FolderItem(["s1", "s2"], "path",
            calculate=False, volume_id="xyzzy")
        self.assertEqual(item.size, 0)
        self.assertEqual(item.path, "path")
        # pylint: disable=E1101
        self.assertFalse(item.thread.started)
        self.assertEqual(item.checkState(0), QtCore.Qt.Unchecked)
        self.assertEqual(item.volume_id, "xyzzy")


class LocalFoldersWithGetSizeTestCase(BaseTestCase):
    """Test the LocalFoldersPage code using the real getsize."""

    class_ui = local_folders.LocalFoldersPage

    def setUp(self):
        """Initialize this test instance."""
        # Create a test folder with a known size
        self.tmpdir = tempfile.mkdtemp()
        f = open(os.path.join(self.tmpdir, 'test_file'), 'wb')
        f.write(" " * 600)
        f.close()
        os.mkdir(os.path.join(self.tmpdir, 'test_dir'))
        f = open(os.path.join(self.tmpdir, 'test_dir', 'test_file_2'), 'wb')
        f.write(" " * 737)
        f.close()
        self.addCleanup(shutil.rmtree, self.tmpdir)
        self.fake_wizard = FakeMainWindow()
        self.patch(local_folders.backend, "ControlBackend", FakeCPBackend)
        super(LocalFoldersWithGetSizeTestCase, self).setUp()
        self.patch(self.ui, "wizard", lambda: self.fake_wizard)

    @defer.inlineCallbacks
    def test_total_size(self):
        """Test that the header reflects the change in item sizes."""
        self.patch(self.ui, "show_hide_offer", self._set_called)
        yield self.ui.get_info()
        self.ui.ui.folder_list.clear()
        self.ui.items = {}
        item = yield self.ui.add_folder(self.tmpdir)
        item.thread.run()
        item.thread.join()
        item.size = 1337
        item.setCheckState(0, QtCore.Qt.Checked)
        self.ui.update_sizes()
        self.assertEqual(unicode(self.ui.ui.folder_list.headerItem().text(1)),
            u"Space (1.5 KiB)")
        self.assertEqual(self._called, ((1537L,), {}))

    def test_size_calculation(self):
        """Test the recursive folder size calculation."""
        queue = Queue.Queue()
        csize = local_folders.CalculateSize(self.tmpdir, queue)
        csize.run()
        path, size = queue.get()
        self.assertEqual(path, self.tmpdir)
        self.assertEqual(size, 1337)


class LocalFoldersTestCase(BaseTestCase):
    """Test the LocalFoldersPage code."""

    class_ui = local_folders.LocalFoldersPage

    def setUp(self):
        """Initialize this test instance."""
        self.tmpdir = tempfile.mkdtemp()
        self.addCleanup(shutil.rmtree, self.tmpdir)
        self.fake_wizard = FakeMainWindow()
        self.patch(local_folders.backend, "ControlBackend", FakeCPBackend)
        self.patch(os.path, 'getsize', lambda *args: 0)
        super(LocalFoldersTestCase, self).setUp()
        self.patch(self.ui, "wizard", lambda: self.fake_wizard)

    def test_status_before_initialize(self):
        """Test status of the page components before initializePage()."""
        self.assertIsInstance(self.ui.timer, QtCore.QTimer)
        self.assertFalse(self.ui.timer.isActive())
        self.assertTrue(self.ui.folders_info is None)
        self.assertTrue(self.ui.account_info is None)
        self.assertEqual(self.ui.items, {})
        self.assertEqual(self.ui.ui.folder_list.topLevelItemCount(), 0)

    def test_status_after_initialize_with_back(self):
        """Test status of page components after initializePage()."""
        self.patch(self.ui, "get_info", self._set_called)
        self.ui.has_back_button = True
        self.ui.initializePage()
        self.assertEqual(self.ui.wizard().overlay.show_counter, 1)
        self.assertEqual(self.ui.wizard().overlay.hide_counter, 0)
        self.assertFalse(self.ui.ui.offer_frame.isVisible())
        self.assertEqual(self._called, ((), {}))
        self.assertEqual(self.ui.wizard().button_texts,
            [(QtGui.QWizard.NextButton, NEXT)])
        self.assertEqual(self.ui.wizard().button_layout,
        [QtGui.QWizard.BackButton,
         QtGui.QWizard.Stretch,
         QtGui.QWizard.NextButton])

    def test_status_after_initialize_without_back(self):
        """Test status of page components after initializePage()."""
        self.patch(self.ui, "get_info", self._set_called)
        self.ui.has_back_button = False
        self.ui.initializePage()
        self.assertEqual(self.ui.wizard().overlay.show_counter, 1)
        self.assertEqual(self.ui.wizard().overlay.hide_counter, 0)
        self.assertFalse(self.ui.ui.offer_frame.isVisible())
        self.assertEqual(self._called, ((), {}))
        self.assertEqual(self.ui.wizard().button_texts,
            [(QtGui.QWizard.NextButton, NEXT)])
        self.assertEqual(self.ui.wizard().button_layout,
        [QtGui.QWizard.Stretch,
         QtGui.QWizard.NextButton])

    @defer.inlineCallbacks
    def test_status_after_get_info(self):
        """Test status of page components after get_info()."""
        yield self.ui.get_info()
        self.assertTrue(self.ui.wizard().overlay.hide_counter, 1)
        self.assertEqual(self.ui.ui.folder_list.topLevelItemCount(), 4)
        self.assertEqual(self.ui.wizard().currentIdChanged.target,
            self.ui.changed_page)
        self.assertEqual(
            self.ui.items[os.path.expanduser('~/xyzzy')].checkState(0),
            QtCore.Qt.Checked)
        check_states = [self.ui.items[item].checkState(0)
            for item in self.ui.items]
        self.assertEqual(len([
            x for x in check_states if x != QtCore.Qt.Unchecked]), 1)
        self.assertNotIn('shared', self.ui.items)

    @defer.inlineCallbacks
    def test_get_info_twice(self):
        """Calling initialize twice should be invariant."""
        yield self.ui.get_info()
        self.assertEqual(self.ui.ui.folder_list.topLevelItemCount(), 4)
        yield self.ui.get_info()
        self.assertEqual(self.ui.ui.folder_list.topLevelItemCount(), 4)

    @defer.inlineCallbacks
    def test_subscribed_udf_checked(self):
        """Check that subscribed UDF items are created correctly."""
        yield self.ui.get_info()
        item = self.ui.items[os.path.expanduser(u'~/xyzzy')]
        self.assertEqual(item.checkState(0), QtCore.Qt.Checked)
        self.assertEqual(item.size, 0)
        self.assertEqual(item.volume_id, 'asdfgh')

    @defer.inlineCallbacks
    def test_unsubscribed_udf_not_added(self):
        """Check that unsubscribed UDF items are not added."""
        yield self.ui.get_info()
        self.assertNotIn(os.path.expanduser(u'~/zxyzzy'), self.ui.items)

    @defer.inlineCallbacks
    def test_item_addition(self):
        """Add a folder."""
        self.ui.ui.folder_list.clear()
        yield self.ui.add_folder(self.tmpdir)
        self.assertEqual(1, self.ui.ui.folder_list.topLevelItemCount())

    @defer.inlineCallbacks
    def test_invalid_item_addition(self):
        """Try to add an invalid folder."""
        self.ui.ui.folder_list.clear()
        self.ui.cp_backend._is_valid = False
        yield self.ui.add_folder(self.tmpdir)
        self.assertEqual(0, self.ui.ui.folder_list.topLevelItemCount())

    @defer.inlineCallbacks
    def test_not_validating_invalid_item_addition(self):
        """Try to add an invalid folder, and not validate it."""
        self.ui.ui.folder_list.clear()
        self.ui.cp_backend._is_valid = False
        yield self.ui.add_folder(self.tmpdir, validate=False)
        self.assertEqual(1, self.ui.ui.folder_list.topLevelItemCount())

    @defer.inlineCallbacks
    def test_not_calculating_item_addition(self):
        """Try to add a folder, and not calculate it."""
        self.ui.ui.folder_list.clear()
        yield self.ui.add_folder(self.tmpdir, calculate=False)
        self.assertEqual(self.ui.items[self.tmpdir].size, 0)
        self.assertEqual(1, self.ui.ui.folder_list.topLevelItemCount())

    @defer.inlineCallbacks
    def test_volume_id_item_addition(self):
        """Try to add a folder with a volume_id."""
        self.ui.ui.folder_list.clear()
        yield self.ui.add_folder(self.tmpdir, volume_id="vol")
        self.assertEqual(self.ui.items[self.tmpdir].volume_id, "vol")
        self.assertEqual(1, self.ui.ui.folder_list.topLevelItemCount())

    @defer.inlineCallbacks
    def test_total_size_unchecked(self):
        """Unchecked items use no space beyond quota_used."""
        self.patch(self.ui, "show_hide_offer", self._set_called)
        yield self.ui.get_info()
        self.ui.ui.folder_list.clear()
        self.ui.items = {}
        item = yield self.ui.add_folder(self.tmpdir)
        item.thread.run()
        item.thread.join()
        item.size = 1337
        item.setCheckState(0, QtCore.Qt.Unchecked)
        self.ui.update_sizes()
        self.assertEqual(unicode(self.ui.ui.folder_list.headerItem().text(1)),
            u"Space (200 bytes)")
        self.assertEqual(self._called, ((200L,), {}))

    @defer.inlineCallbacks
    def test_total_size_unchecked_no_udfs(self):
        """No checked items and no quota used should amount to 0 bytes."""
        self.patch(self.ui, "show_hide_offer", self._set_called)
        yield self.ui.get_info()
        self.ui.account_info['quota_used'] = 0
        self.ui.ui.folder_list.clear()
        self.ui.items = {}
        item = yield self.ui.add_folder(self.tmpdir)
        item.thread.run()
        item.thread.join()
        item.size = 1337
        item.setCheckState(0, QtCore.Qt.Unchecked)
        self.ui.update_sizes()
        self.assertEqual(unicode(self.ui.ui.folder_list.headerItem().text(1)),
            u"Space (0 bytes)")
        self.assertEqual(self._called, ((0,), {}))

    @defer.inlineCallbacks
    def test_total_size_udf(self):
        """UDFs use no space beyond quota_used."""
        self.patch(self.ui, "show_hide_offer", self._set_called)
        yield self.ui.get_info()
        for _, item in self.ui.items.items():
            if item.thread.is_alive():
                item.thread.join()
            if item.volume_id:
                item.setCheckState(0, QtCore.Qt.Checked)
            else:
                item.setCheckState(0, QtCore.Qt.Unchecked)
        self.ui.update_sizes()
        self.assertEqual(unicode(self.ui.ui.folder_list.headerItem().text(1)),
            u"Space (200 bytes)")
        self.assertEqual(self._called, ((200L,), {}))

    def test_add_twice(self):
        """Behaviour for adding the same folder twice:

        * It's added only once.
        """
        self.ui.ui.folder_list.clear()
        self.assertEqual(0, self.ui.ui.folder_list.topLevelItemCount())
        self.ui.add_folder(self.tmpdir)
        self.ui.add_folder(self.tmpdir)
        self.assertEqual(1, self.ui.ui.folder_list.topLevelItemCount())

    @defer.inlineCallbacks
    def test_add_missing_folder(self):
        """Behaviour for adding a folder that doesn't exist:

        * It's added.
        * Has size 0.
        """

        yield self.ui.get_info()
        self.ui.ui.folder_list.clear()
        self.assertEqual(0, self.ui.ui.folder_list.topLevelItemCount())
        item = yield self.ui.add_folder(os.path.join(
            "xyzzy", "xyzzy", "xyzzy"))
        self.assertEqual(1, self.ui.ui.folder_list.topLevelItemCount())
        item.thread.run()
        item.thread.join()
        self.ui.update_sizes()
        self.assertEqual(0, item.size)
        # world did not explode

    def test_over_quota(self):
        """After removing all folders, offer_frame should be hidden.

        Push the user over quota, it should be visible.
        """
        self.patch(self.ui.ui.offer_frame, "setVisible", self._set_called)
        yield self.ui.get_info()
        self.ui.ui.folder_list.clear()
        self.ui.update_sizes()
        self.assertEqual(self._called, ((False,), {}))
        self.ui.show_hide_offer(self.ui.quota() + 1)
        self.assertEqual(self._called, ((True,), {}))

    def test_add_folder_clicked_valid(self):
        """Test behaviour when adding a valid folder via the button."""
        self.patch(QtGui, "QFileDialog", FakeFileDialog())
        self.patch(self.ui, "add_folder", self._set_called)
        self.ui.ui.add_folder_button.click()
        self.assertEqual(self._called,
            ((u'c:\\whatever',), {'validate': False, 'volume_id': False}))

    @defer.inlineCallbacks
    def test_add_folder_valid_starts_selected(self):
        """When adding a valid folder, it starts as selected."""
        self.patch(self.ui, 'update_sizes', lambda: None)
        self.patch(QtGui, "QFileDialog", FakeFileDialog())
        yield self.ui.on_add_folder_button_clicked()
        self.assertEqual(
            self.ui.items[u'c:\\whatever'].checkState(0),
            QtCore.Qt.Checked)

    def test_add_folder_clicked_invalid(self):
        """Test behaviour when adding an invalid folder via the button."""
        self.patch(QtGui, "QFileDialog", FakeFileDialog())
        message_box = FakeMessageBox()
        self.patch(QtGui, "QMessageBox", message_box)
        self.patch(self.ui, "add_folder", self._set_called)
        self.ui.cp_backend._is_valid = False
        self.ui.ui.add_folder_button.click()
        self.assertEqual(self._called, False)
        user_home = os.path.expanduser('~')
        text = local_folders.FOLDER_INVALID_PATH % {
            'folder_path': u"c:\\whatever",
            'home_folder': user_home,
        }
        # pylint: disable=W0212
        self.assertEqual(message_box._warning,
            ((self.ui,
             '', text, 2), {}))

    @defer.inlineCallbacks
    def test_changed_page_existing_udf_behaviour(self):
        """If a UDF is checked, subscribe it, if not, unsubscribe it."""
        yield self.ui.get_info()
        show_counter = self.ui.wizard().overlay.show_counter
        hide_counter = self.ui.wizard().overlay.hide_counter
        self.ui.changed_page(self.ui.wizard().SYNC_NOW_OR_LATER_PAGE)
        changes = sorted(self.ui.cp_backend.volume_setings_changes)
        self.assertEqual(changes,
            [('asdfgh', {'subscribed': True})])
        self.assertEqual(self.ui.wizard().overlay.show_counter,
            show_counter + 1)
        self.assertEqual(self.ui.wizard().overlay.hide_counter,
            hide_counter + 1)

    @defer.inlineCallbacks
    def test_changed_page_new_udf_behaviour(self):
        """Create UDFs for non-existing, checked UDFs."""
        yield self.ui.get_info()
        show_counter = self.ui.wizard().overlay.show_counter
        hide_counter = self.ui.wizard().overlay.hide_counter
        self.ui.ui.folder_list.clear()
        self.ui.items = {}
        item = yield self.ui.add_folder("whatever")
        item.setCheckState(0, QtCore.Qt.Checked)
        item = yield self.ui.add_folder("whatever2")
        item.setCheckState(0, QtCore.Qt.Unchecked)
        self.ui.changed_page(self.ui.wizard().SYNC_NOW_OR_LATER_PAGE)
        self.assertEqual(self.ui.cp_backend.folders_created,
            [('whatever',)])
        self.assertEqual(self.ui.wizard().overlay.show_counter,
            show_counter + 1)
        self.assertEqual(self.ui.wizard().overlay.hide_counter,
            hide_counter + 1)

    def test_exception_on_account_info(self):
        """When account_info fails, nothing should happen."""
        self.patch(self.ui,
            "cp_backend", FakeFailingCPBackend())
        self.ui.get_info()
        # Still here

    def test_timer_is_started(self):
        """When displaying the page, the timer should start."""
        self.ui.initializePage()
        self.assertTrue(self.ui.timer.isActive())

    def test_timer_is_stopped(self):
        """When leaving the page, the timer should stop."""
        self.ui.changed_page(-1)
        self.assertFalse(self.ui.timer.isActive())

    def test_on_folder_list_item_changed_col_0(self):
        """Test that it calls the right thing."""
        self.patch(self.ui, 'update_sizes', self._set_called)
        self.ui.on_folder_list_itemChanged("item", 0)
        self.assertEqual(self._called, ((), {}))

    def test_on_folder_list_item_changed_col_1(self):
        """Test that it calls nothing."""
        self.patch(self.ui, 'update_sizes', self._set_called)
        self.ui.on_folder_list_itemChanged("item", 1)
        self.assertFalse(self._called)

    @defer.inlineCallbacks
    def test_runtime_error(self):
        """Test that runtime error is handled correctly."""

        def raise_runtime(*args):
            """Dummy function that raises RuntimeError."""
            raise RuntimeError

        yield self.ui.get_info()
        item = yield self.ui.add_folder(
            "dummy", validate=False, calculate=False)
        self.patch(item, 'setText', raise_runtime)
        self.ui.queue.put(["dummy", 1000])
        self.assertIn("dummy", self.ui.items)
        self.ui.update_sizes()
        self.assertNotIn("dummy", self.ui.items)

    @defer.inlineCallbacks
    def test_queue_is_emptied(self):
        """Test that update_sizes empties the queue."""
        self.ui.account_info = {
            "quota_total": 1000,
            "quota_used": 200,
        }
        yield self.ui.add_folder("dummy1", validate=False, calculate=False)
        yield self.ui.add_folder("dummy2", validate=False, calculate=False)
        self.ui.queue.put(["dummy1", 1000])
        self.ui.queue.put(["dummy2", 1000])
        self.assertFalse(self.ui.queue.empty())
        self.ui.update_sizes()
        self.assertTrue(self.ui.queue.empty())

    @defer.inlineCallbacks
    def test_not_native(self):
        """Test that we are *not* using the native file dialog."""
        self.patch(local_folders.QtGui.QFileDialog,
            "getExistingDirectory", self._set_called)
        try:
            yield self.ui.on_add_folder_button_clicked()
        except TypeError:
            pass  # Expected exception
        self.assertEqual(self._called, ((), {
            'options': local_folders.QtGui.QFileDialog.DontUseNativeDialog,
            'parent': self.ui,
        }))

    def test_cleanup_page(self):
        """cleanupPage should call folders.initializePage()."""
        self.patch(self.ui.wizard().folders_page,
            "initializePage", self._set_called)
        self.ui.cleanupPage()
        self.assertEqual(self._called, ((), {}))
