# -*- coding: utf-8 -*-

# Authors: Alejandro J. Cura <alecu@canonical.com>
#          Roberto Alsina <roberto.alsina@canonical.com>
#          Diego Sarmentero <diego.sarmentero@canonical.com>
#
# Copyright 2011 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Tests for the Setup Account page Qt UI."""

from PyQt4 import QtGui, QtCore

from ubuntu_sso.qt import common

from ubuntuone_installer.gui.qt import setup_account as gui
from ubuntuone_installer.gui.qt.tests import (
    BaseTestCase,
    FakeController,
    FakeWizard,
)
from ubuntuone_installer.gui.qt.ui import setup_account_ui


class SetupAccountTestCase(BaseTestCase):
    """Test the SetupAccountPage code."""

    class_ui = gui.SetupAccountPage
    kwargs = dict(
        ui=setup_account_ui.Ui_SetUpAccountPage(),
        controller=FakeController(),
        parent=None,
    )

    def setUp(self):
        super(SetupAccountTestCase, self).setUp()
        self.patch(self.ui, "set_up_button", QtGui.QPushButton())

    def test_init(self):
        """Check the initial state of SetupAccount."""
        self.assertEqual(self.ui.ui.password_edit.receivers(
            QtCore.SIGNAL('textEdited(QString)')), 1)

    def test_empty_name(self):
        """Status when the name field is empty.

        * Name Assistance label visible.
        * Correct error message in it.
        """
        self.ui.ui.name_edit.setText("")
        self.ui.name_assistance()
        self.ui.show()
        self.addCleanup(self.ui.hide)
        self.assertTrue(self.ui.ui.name_assistance.isVisible())
        self.assertEqual(
            unicode(self.ui.ui.name_assistance.text()),
            gui.ERROR % gui.EMPTY_NAME)
        self.ui.hide()

    def test_password_default_assistance(self):
        """Status when the password line edit receive focus and shows popup.

        * Password assistance contains the right message.
        """
        self.ui.ui.password_edit.setText("")
        self.ui.ui.confirm_password_edit.setText("")
        common.password_default_assistance(self.ui.ui.password_assistance)
        self.ui.focus_changed(None, self.ui.ui.password_edit)
        self.assertIn(
            common.NORMAL % common.PASSWORD_LENGTH,
            unicode(self.ui.ui.password_assistance.text()),
        )
        self.assertIn(
            common.NORMAL % common.PASSWORD_UPPER,
            unicode(self.ui.ui.password_assistance.text()),
        )
        self.assertIn(
            common.NORMAL % common.PASSWORD_DIGIT,
            unicode(self.ui.ui.password_assistance.text()),
        )

    def test_password_focus_gain(self):
        """Check functions execution when focus_changed() is executed."""
        self.ui.show()
        self.addCleanup(self.ui.hide)
        self.ui.ui.password_assistance.setVisible(False)
        self.assertFalse(self.ui.ui.password_assistance.isVisible())
        self.patch(self.ui, 'name_assistance', self._set_called)
        self.ui.focus_changed(None, self.ui.ui.password_edit)
        self.assertTrue(self.ui.ui.password_assistance.isVisible())

    def test_name_focus_lost(self):
        """Check functions execution when focus_changed() is executed."""
        self.patch(self.ui, 'name_assistance', self._set_called)
        self.ui.focus_changed(self.ui.ui.name_edit, None)
        self.assertTrue(self._called)

    def test_email_focus_lost(self):
        """Check functions execution when focus_changed() is executed."""
        self.patch(self.ui, 'email_assistance', self._set_called)
        self.ui.focus_changed(self.ui.ui.email_edit, None)
        self.assertTrue(self._called)

    def test_confirm_email_focus_lost(self):
        """Check functions execution when focus_changed() is executed."""
        self.patch(self.ui, 'confirm_email_assistance', self._set_called)
        self.ui.focus_changed(self.ui.ui.confirm_email_edit, None)
        self.assertTrue(self._called)

    def test_confirm_password_focus_lost(self):
        """Check functions execution when focus_changed() is executed."""
        self.patch(common, 'password_check_match', self._set_called)
        self.ui.focus_changed(self.ui.ui.confirm_password_edit, None)
        self.assertTrue(self._called)


class SetupAccountFakeWizardTestCase(BaseTestCase):
    """Test the SetupAccountPage code."""

    class_ui = gui.SetupAccountPage
    kwargs = dict(
        ui=setup_account_ui.Ui_SetUpAccountPage(),
        controller=FakeController(),
        parent=None,
    )

    def setUp(self):
        """Initialize this test instance."""
        # Faking each SSO object instead of doing it lower
        # so we don't rely on any SSO behaviour
        super(SetupAccountFakeWizardTestCase, self).setUp()
        self.wizard = FakeWizard()
        self.patch(self.ui, 'wizard', lambda: self.wizard)

    def test_initialize_page(self):
        """Widgets are properly initialized."""
        self.ui.initializePage()
        self.ui.show()
        self.addCleanup(self.ui.hide)

        # titles
        self.assertEqual(self.ui.header.title_label.text(),
                         gui.TITLE_STYLE % gui.TITLE)
        self.assertEqual(self.ui.header.subtitle_label.text(), gui.SUBTITLE)

        # set up account button
        expected = [QtGui.QWizard.BackButton, QtGui.QWizard.Stretch,
                    QtGui.QWizard.CustomButton3]
        self.assertIn(('setButtonLayout', ((expected,), {})),
                       self.ui.wizard().called)
        self.assertEqual(gui.SET_UP_ACCOUNT_BUTTON,
                         self.ui.buttonText(QtGui.QWizard.CustomButton3))
        self.assertTrue(self.ui.wizard().button(QtGui.QWizard.CustomButton3) is
                        self.ui.set_up_button)
        self.assertFalse(self.ui.set_up_button.isEnabled())

        self.assertFalse(self.ui.captcha_received)

        # labels
        self.assertEqual(self.ui.ui.name_label.text(), gui.NAME)
        self.assertEqual(self.ui.ui.email_label.text(), gui.EMAIL)
        self.assertEqual(self.ui.ui.confirm_email_label.text(),
                         gui.RETYPE_EMAIL)
        self.assertEqual(self.ui.ui.password_label.text(),
                         gui.PASSWORD)
        self.assertEqual(self.ui.ui.confirm_password_label.text(),
                         gui.RETYPE_PASSWORD)
        self.assertFalse(self.ui.ui.password_info_label.isVisible())

        # assistants
        self.assertFalse(self.ui.ui.name_assistance.isVisible())
        self.assertFalse(self.ui.ui.email_assistance.isVisible())
        self.assertFalse(self.ui.ui.confirm_email_assistance.isVisible())
        self.assertFalse(self.ui.ui.password_assistance.isVisible())
        self.assertTrue(self.ui.ui.refresh_label.isVisible())

    def test_set_up_button_clicked(self):
        """Validation and overlay is shown when the setupbutton is clicked."""
        self.patch(self.ui.controller, 'set_next_validation', self._set_called)
        self.ui.initializePage()
        self.ui.captcha_received = True
        self.ui.show()
        self.addCleanup(self.ui.hide)

        self.ui.set_up_button.clicked.emit(False)

        self.assertEqual(self.ui.wizard().overlay.show_counter, 1)
        self.assertEqual(self._called, ((False,), {}))

    def test_set_error_message(self):
        """Check the state of the label after calling: set_error_message."""
        self.ui.email_assistance()
        self.ui.show()
        self.addCleanup(self.ui.hide)
        self.ui.set_error_message(self.ui.ui.email_assistance, "message")
        self.assertTrue(self.ui.ui.email_assistance.isVisible())
        self.assertEqual(
            unicode(self.ui.ui.email_assistance.text()),
            gui.ERROR % "message")

    def test_blank_name(self):
        """Status when the name field is blank (spaces).

        * Name Assistance label visible.
        * Correct error message in it.
        """
        self.ui.ui.name_edit.setText("  ")
        self.ui.name_assistance()
        self.ui.show()
        self.addCleanup(self.ui.hide)
        self.assertTrue(self.ui.ui.name_assistance.isVisible())
        self.assertEqual(
            unicode(self.ui.ui.name_assistance.text()),
            gui.ERROR % gui.EMPTY_NAME)
        self.ui.hide()

    def test_valid_name(self):
        """Status when the name field is valid.

        * Name Assistance label invisible.
        """
        self.ui.ui.name_edit.setText("John Doe")
        self.ui.name_assistance()
        self.ui.show()
        self.addCleanup(self.ui.hide)
        self.assertFalse(self.ui.ui.name_assistance.isVisible())
        self.ui.hide()

    def test_invalid_email(self):
        """Status when the email field has no @.

        * Email Assistance label visible.
        * Correct error message in it.
        """
        self.ui.ui.email_edit.setText("foobar")
        self.ui.email_assistance()
        self.ui.show()
        self.addCleanup(self.ui.hide)
        self.assertTrue(self.ui.ui.email_assistance.isVisible())
        self.assertEqual(
            unicode(self.ui.ui.email_assistance.text()),
            gui.ERROR % gui.INVALID_EMAIL)

    def test_valid_email(self):
        """Status when the email field has a @.

        * Email Assistance label invisible.
        """
        self.ui.ui.email_edit.setText("foo@bar")
        self.ui.email_assistance()
        self.ui.show()
        self.addCleanup(self.ui.hide)
        self.assertFalse(self.ui.ui.email_assistance.isVisible())
        self.ui.hide()

    def test_matching_emails(self):
        """Status when the email fields match.

        * Email Assistance label invisible.
        """
        self.ui.ui.email_edit.setText("foo@bar")
        self.ui.ui.confirm_email_edit.setText("foo@bar")
        self.ui.confirm_email_assistance()
        self.ui.show()
        self.addCleanup(self.ui.hide)
        self.assertFalse(self.ui.ui.confirm_email_assistance.isVisible())
        self.ui.hide()

    def test_not_matching_emails(self):
        """Status when the email fields don't match.

        * Email Assistance label visible.
        * Correct error message.
        """
        self.ui.ui.email_edit.setText("foo@bar")
        self.ui.ui.confirm_email_edit.setText("foo@baz")
        self.ui.confirm_email_assistance()
        self.ui.show()
        self.addCleanup(self.ui.hide)
        self.assertTrue(self.ui.ui.confirm_email_assistance.isVisible())
        self.assertEqual(
            unicode(self.ui.ui.confirm_email_assistance.text()),
            gui.ERROR % gui.EMAIL_MATCH)
        self.ui.hide()

    def test_focus_changed_password_visibility(self):
        """Check visibility changes when focus_changed() is executed."""
        self.ui.show()
        self.addCleanup(self.ui.hide)
        self.ui.focus_changed(None, self.ui.ui.password_edit)
        self.assertTrue(self.ui.ui.password_assistance.isVisible())

    def test_hide_event(self):
        """Check that hide_event works when set_up_button is none."""
        self.assertEqual(self.ui.set_up_button, None)
        self.ui.hideEvent(QtGui.QHideEvent())

    def test_show_event(self):
        """Check that show_event works when set_up_button is none."""
        self.assertEqual(self.ui.set_up_button, None)
        self.ui.showEvent(QtGui.QShowEvent())

    def test_on_captcha_refreshing(self):
        """Check the state of the overlay on captcha refreshing."""
        self.assertEqual(self.ui.wizard().overlay.show_counter, 0)
        self.ui.on_captcha_refreshing()
        self.assertEqual(self.ui.wizard().overlay.show_counter, 0)
        self.ui.captcha_received = True
        self.ui.show()
        self.addCleanup(self.ui.hide)
        self.assertEqual(self.ui.wizard().overlay.show_counter, 0)
        self.ui.on_captcha_refreshing()
        self.assertEqual(self.ui.wizard().overlay.show_counter, 1)

    def test_on_captcha_refresh_complete(self):
        """Check the state of the overlay on captcha refreshing complete."""
        self.assertEqual(self.ui.wizard().overlay.hide_counter, 0)
        self.ui.on_captcha_refresh_complete()
        self.assertEqual(self.ui.wizard().overlay.hide_counter, 1)
