# -*- coding: utf-8 -*-

# Authors: Diego Sarmentero <diego.sarmentero@canonical.com>
#          Manuel de la Pena <manuel@canonical.com>
#
# Copyright 2011 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Tests for the platform specific functions."""

import sys
import os
import subprocess

# Avoid pylint error on Linux
# pylint: disable=F0401
import win32api
# pylint: enable=F0401

from twisted.internet import defer

from ubuntuone_installer.gui.qt import utils
from ubuntuone_installer.gui.qt.tests import BaseTestCase


class UninstallerTestCase(BaseTestCase):

    """Test the uninstaller."""

    def test_execute_uninstall_on_licence_cancel_file_not_exists(self):
        """Pressing Disagree button from license page the uninstall is exec."""
        # If it's frozen this test fails because the path
        # depends on the location of the exe
        frozen = getattr(sys, "frozen", None)
        if frozen is not None:
            delattr(sys, "frozen")
            self.addCleanup(setattr, sys, 'frozen', frozen)
        self.patch(win32api, "ShellExecute", self._set_called)
        self.assertFalse(self._called)

    def test_execute_uninstall_on_licence_cancel_frozen_file_not_exists(self):
        """Pressing Disagree button from license page the uninstall is exec."""
        self.patch(win32api, "ShellExecute", self._set_called)
        sys.frozen = True
        self.addCleanup(delattr, sys, "frozen")
        utils.uninstall_application()
        self.assertFalse(self._called)

    def test_execute_uninstall_on_licence_cancel(self):
        """Pressing Disagree button from license page the uninstall is exec."""
        self.patch(win32api, "ShellExecute", self._set_called)
        self.patch(os.path, "exists", lambda path: True)
        utils.uninstall_application()
        # If it's frozen this test fails because the path
        # depends on the location of the exe
        frozen = getattr(sys, "frozen", None)
        if frozen is not None:
            delattr(sys, "frozen")
            self.addCleanup(setattr, sys, 'frozen', frozen)
        exec_path = os.path.abspath(__file__)
        folder_path_utils_dir = os.path.dirname(exec_path)
        folder_path_qt_dir = os.path.dirname(folder_path_utils_dir)
        folder = os.path.dirname(folder_path_qt_dir)
        uninstall_path = os.path.join(os.path.dirname(folder), "uninstall.exe")
        self.assertEqual(self._called,
            ((None, 'runas', uninstall_path, '--mode win32', '', 0), {}))

    def test_execute_uninstall_on_licence_cancel_frozen(self):
        """Pressing Disagree button from license page the uninstall is exec."""
        self.patch(win32api, "ShellExecute", self._set_called)
        self.patch(os.path, "exists", lambda path: True)
        sys.frozen = True
        self.addCleanup(delattr, sys, "frozen")
        utils.uninstall_application()
        self.assertTrue(self._called[0][0] is None)
        self.assertIn("uninstall.exe", self._called[0][2])
        self.assertEqual('runas', self._called[0][1])
        self.assertEqual('--mode win32', self._called[0][3])
        self.assertEqual('', self._called[0][4])
        self.assertEqual(0, self._called[0][5])


class StartControlPanelTestCase(BaseTestCase):

    """Test the start of the control panel."""

    def test_start_control_panel_with_icon(self):
        """The proper process is started."""
        self.patch(subprocess, "Popen", self._set_called)
        utils.start_control_panel()
        expected = ((['ubuntuone-control-panel-qt', '--with-icon'],), {})
        self.assertEqual(self._called, expected)

    def test_start_control_panel_without_icon(self):
        """The proper process is started."""
        self.patch(subprocess, "Popen", self._set_called)
        utils.start_control_panel(with_icon=False)
        expected = ((['ubuntuone-control-panel-qt'],), {})
        self.assertEqual(self._called, expected)

    def test_start_control_panel_frozen(self):
        """When frozen, the control-panel has a path."""
        # I can't patch sys because frozen is not there by default
        sys.frozen = True
        self.addCleanup(delattr, sys, 'frozen')

        root = 'Z:\\Yadda\\Yoo'
        self.patch(os.path, 'abspath', lambda p: os.path.join(root, p))
        self.patch(os.path, 'dirname', lambda p: p)
        exe = 'foo\\bar\\baz\\'
        self.patch(sys, 'executable', exe)

        self.patch(subprocess, "Popen", self._set_called)

        utils.start_control_panel()

        path = os.path.join(root, exe, 'ubuntuone-control-panel-qt.exe')
        expected = (([path, '--with-icon'],), {})
        self.assertEqual(self._called, expected)


class FakeRegistry(object):

    """A fake registry."""

    # pylint: disable=C0103
    HKEY_CURRENT_USER = 2
    KEY_ALL_ACCESS = 4
    REG_SZ = 1

    def __init__(self):
        self.HKEY_CURRENT_USER = 2
        self.KEY_ALL_ACCESS = 4
        self.connect_args = None
        self.openkey_args = None
        self.closekey_args = None
        self.query_args = None
        self.set_args = []

    def ConnectRegistry(self, *args, **kwargs):
        """Fake ConnectRegistry."""
        self.connect_args = (args, kwargs)
        return "REGISTRY"

    def OpenKey(self, *args, **kwargs):
        """Fake OpenKey."""
        self.openkey_args = (args, kwargs)
        return "KEY"

    def CloseKey(self, *args, **kwargs):
        """Fake CloseKey."""
        self.closekey_args = (args, kwargs)

    def QueryValueEx(self, *args, **kwargs):
        """Fake QueryValueEx."""
        self.query_args = (args, kwargs)

    def SetValueEx(self, *args, **kwargs):
        """Fake SetValueEx."""
        self.set_args.append((args, kwargs))


class AutostartTestCase(BaseTestCase):

    """Test add_syncdaemon_to_autostart."""

    def setUp(self):
        """Initialize this test instance."""
        self.registry = FakeRegistry()
        self.patch(utils.windows, "_winreg", self.registry)
        super(AutostartTestCase, self).setUp()

    def test_add_syncdaemon_to_autostart(self):
        """Check that the registry is updated correctly."""
        # I can't patch sys because frozen is not there by default
        sys.frozen = True
        self.addCleanup(delattr, sys, 'frozen')
        utils.windows.add_syncdaemon_to_autostart()
        self.assertEqual(self.registry.connect_args,
            ((None, self.registry.HKEY_CURRENT_USER), {}))
        self.assertEqual(self.registry.openkey_args,
            (('REGISTRY', utils.windows.AUTORUN_KEY, 0,
              self.registry.KEY_ALL_ACCESS), {}))
        self.assertEqual(self.registry.closekey_args, (('KEY',), {}))
        self.assertEqual(self.registry.query_args, None)
        self.assertEqual(self.registry.set_args,
             [(('KEY', 'Ubuntu One', 0, 1,
             'C:\\Python27\\ubuntuone-syncdaemon.exe'), {}),
             (('KEY', 'Ubuntu One Icon', 0, 1,
             'C:\\Python27\\ubuntuone-control-panel-qt.exe --minimized'),
             {})])

    def test_not_added_if_not_frozen(self):
        """Not frozen binaries are not added to the registry."""
        utils.windows.add_syncdaemon_to_autostart()
        self.assertEqual(self.registry.connect_args, None)
        self.assertEqual(self.registry.openkey_args, None)
        self.assertEqual(self.registry.closekey_args, None)
        self.assertEqual(self.registry.query_args, None)
        self.assertEqual(self.registry.set_args, [])


class DefaultFoldersTestCase(BaseTestCase):

    """Test the default suggested UDFs."""

    def test_special_folders(self):
        """Test that default_folders does the right thing."""
        folders = utils.default_folders()
        self.assertEqual(len(folders), 3)
        self.assertEqual(folders, [os.path.abspath(f) for f in folders])
        # pylint: disable=W0404
        import ctypes
        dll = ctypes.windll.shell32
        buf = ctypes.create_string_buffer(300)
        dll.SHGetSpecialFolderPathA(None, buf, 5, False)
        docs = buf.value
        dll.SHGetSpecialFolderPathA(None, buf, 13, False)
        music = buf.value
        dll.SHGetSpecialFolderPathA(None, buf, 39, False)
        pictures = buf.value
        expected = [docs, music, pictures]
        self.assertEqual(sorted(folders), sorted(expected))


class OldBetaUpdateTestCase(BaseTestCase):

    """Test the update from the old beta."""


    def _assert_is_old_beta_installed(self, return_code, expected_result):
        """Helper to test the is_old_beta_installed."""
        # vars used to know that we did indeed call the msi lib correctly
        called = {}

        def fake_get_product_info(uid, property, property_buffer, size):
            """Fake call to the msi lib."""
            called['uid'] = uid
            called['property'] = property
            called['property_buffer'] = property_buffer
            called['size'] = size
            return return_code

        self.patch(utils.windows.ctypes.windll.msi, 'MsiGetProductInfoW',
                   fake_get_product_info)
        self.assertEqual(utils.is_old_beta_installed(), expected_result)
        # assert that we indeed called the ctypes with the correct data
        self.assertEqual(utils.windows.OLD_BETA_UID_KEY,
                         called['uid'].value)
        self.assertEqual(utils.windows.VERSION_STRING_PROPERTY, called['property'])

    def test_is_old_beta_installed_true(self):
        """Test the function used to know if it is installed."""
        self._assert_is_old_beta_installed(utils.windows.ERROR_SUCCESS, True)

    def test_is_old_beta_installed_more_data(self):
        """Test the function used to know if it is installed."""
        self._assert_is_old_beta_installed(utils.windows.ERROR_MORE_DATA, True)

    def test_is_old_beta_installed_false(self):
        """Test the function used to know if it is installed."""
        self._assert_is_old_beta_installed(utils.windows.ERROR_UNKNOWN_PRODUCT,
                False)

    def _assert_get_product_property(self, uid, property, return_code, expected_result):
        """Generic test for getting the property."""
        # vars used to know that we did indeed call the msi lib correctly
        self.called_uid_buffer = None
        self.called_property_name = None
        self.called_property_buffer = None
        self.called_size = None

        def fake_get_product_info(uid, property, property_buffer, size):
            """Fake call to the msi lib."""
            self.called_uid_buffer = uid
            self.called_property_name = property
            if expected_result:
                property_buffer.value = expected_result
                self.called_property_buffer = expected_result
            self.called_size = size
            return return_code

        self.patch(utils.windows.ctypes.windll.msi, 'MsiGetProductInfoW',
                   fake_get_product_info)
        self.assertEqual(expected_result,
                         utils.windows.get_property_for_product(uid, property))
        self.assertEqual(uid, self.called_uid_buffer, uid)
        self.assertEqual(property, self.called_property_name)


    def test_get_product_property_present(self):
        """Test that we do indeed get the correct value."""
        uid = u'{som-uid}'
        property = u'VersionString'
        return_code = 0
        self._assert_get_product_property(uid, property, return_code, property)

    def test_get_product_property_not_present(self):
        """Test that we do get None."""
        uid = u'{som-uid}'
        property = u'VersionString'
        return_code = 1212
        self._assert_get_product_property(uid, property, return_code, None)

    def test_uninstall_present(self):
        """Test uninstalling when the msi is present."""
        self.property_uid = None
        self.install_path = None
        self.property = None
        self.callable = None
        self.cmd = None
        self.path = 'path/to/msi'

        def fake_get_property_for_product(uid, property, buf_size=None):
            """A fake method."""
            self.property_uid = uid
            self.property = property
            return self.path

        def fake_msi_install_product(callable, path, cmd):
            """A fake install function."""
            self.callable = callable
            self.install_path = path
            self.cmd = cmd
            return defer.succeed(0)

        self.patch(utils.windows, 'get_property_for_product',
                   fake_get_property_for_product)
        self.patch(utils.windows, 'deferToThread',
                   fake_msi_install_product)
        utils.uninstall_old_beta()
        self.assertEqual(self.property_uid, utils.windows.OLD_BETA_UID_KEY)
        self.assertEqual(self.property, u'LocalPackage')
        self.assertEqual(utils.windows.ctypes.windll.msi.MsiInstallProductW,
                         self.callable)
        self.assertEqual(self.install_path, self.path)
        self.assertEqual(self.cmd, u'REMOVE=ALL')

    def test_uninstall_error(self):
        """Test uninstalling with an error."""
        self.property_uid = None
        self.install_path = None
        self.property = None
        self.callable = None
        self.cmd = None
        self.path = 'path/to/msi'

        def fake_get_property_for_product(uid, property, buf_size=None):
            """A fake method."""
            self.property_uid = uid
            self.property = property
            return self.path

        def fake_msi_install_product(callable, path, cmd):
            """A fake install function."""
            self.callable = callable
            self.install_path = path
            self.cmd = cmd
            return defer.succeed(1)

        self.patch(utils.windows, 'get_property_for_product',
                   fake_get_property_for_product)
        self.patch(utils.windows, 'deferToThread',
                   fake_msi_install_product)
        self.assertFailure(utils.uninstall_old_beta(),
                           utils.windows.MsiException)
        self.assertEqual(self.property_uid, utils.windows.OLD_BETA_UID_KEY)
        self.assertEqual(self.property, u'LocalPackage')
        self.assertEqual(utils.windows.ctypes.windll.msi.MsiInstallProductW,
                         self.callable)
        self.assertEqual(self.install_path, self.path)
        self.assertEqual(self.cmd, u'REMOVE=ALL')

    def test_uninstall_not_present(self):
        """Test uninstalling when the msi is missing."""
        self.uid = None
        self.property = None

        def fake_get_property_for_product(uid, property):
            """A fake method."""
            self.uid = uid
            self.property = property
            return None

        self.patch(utils.windows, 'get_property_for_product',
                   fake_get_property_for_product)
        self.assertFailure(utils.uninstall_old_beta(),
                           utils.windows.MsiException)
        self.assertEqual(self.uid, utils.windows.OLD_BETA_UID_KEY)
        self.assertEqual(self.property, u'LocalPackage')


class FakeLogger(object):
    """A Fake logging object."""

    def __init__(self):
        """Create a new instance."""
        self.called_methods = []

    def info(self, message, *args, **kwargs):
        """Set an info log message."""
        self.called_methods.append(('info', message, args, kwargs))

    def debug(self, message, *args, **kwargs):
        """Set a debug log message."""
        self.called_methods.append(('debug', message, args, kwargs))


class AutoupdaterTestCase(BaseTestCase):
    """Test the code that is used for the auto update process."""

    @defer.inlineCallbacks
    def setUp(self):
        """Prepare for the diff tests."""
        yield super(AutoupdaterTestCase, self).setUp()
        self.auto_update_path = r'path\to\exe'
        self.return_from_call = 0
        self.command = None
        self.args = []

        def fake_execute_process(command, args=None, path=None):
            """Fake async process execution."""
            self.command = command
            self.args = args
            return self.return_from_call

        self.fake_logger = FakeLogger()
        self.patch(utils.windows, 'getProcessValue', fake_execute_process)
        self.patch(utils.windows, '_get_update_path',
                   lambda: self.auto_update_path)

    @defer.inlineCallbacks
    def test_are_updates_present_true(self):
        """Test when updates are present."""
        # the idea is simple, set the value to be returned from
        # the fake call, assert that we get true and also that
        # we did use the correct parameters.
        self.return_from_call = 0
        are_present = yield utils.are_updates_present(self.fake_logger)
        self.assertTrue(are_present, 'Updates should be present.')
        # lets assert that we did use the correct args
        expected_args = ('--mode', 'unattended')
        self.assertEqual(expected_args, self.args)
        self.assertEqual(self.command, self.auto_update_path)

    @defer.inlineCallbacks
    def test_are_updates_present_false(self):
        """Test when updates are not present."""
        # similar to test_are_updates_present_true but with diff retcode
        self.return_from_call = 40
        are_present = yield utils.are_updates_present(self.fake_logger)
        self.assertFalse(are_present, 'Updates should NOT be present.')
        # lets assert that we did use the correct args
        expected_args = ('--mode', 'unattended')
        self.assertEqual(expected_args, self.args)
        self.assertEqual(self.command, self.auto_update_path)

    def test_perform_update(self):
        """Test the method that perfroms the update."""
        self.patch(utils.windows.win32api, 'ShellExecute', self._set_called)
        utils.perform_update()
        self.assertIn(self.auto_update_path, self._called[0][2])
        self.assertEqual('runas', self._called[0][1])
        self.assertEqual('--unattendedmodeui none', self._called[0][3])
        self.assertEqual('', self._called[0][4])
        self.assertEqual(0, self._called[0][5])

    def test_get_auto_update_path(self):
        """Test the method used to get the autoupdate."""
