# -*- coding: utf-8 -*-

# Authors: Diego Sarmentero <diego.sarmentero@canonical.com>
#          Manuel de la Pena <manuel@canonical.com>
#
# Copyright 2011 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Utils Functions Windows specific."""


import ctypes
import os
import subprocess
import sys

# Avoid pylint error on Linux
# pylint: disable=F0401
import win32api
import _winreg

from ctypes import wintypes
# pylint: enable=F0401

from twisted.internet import defer
from twisted.internet.utils import getProcessValue
from twisted.internet.threads import deferToThread

AUTORUN_KEY = r"Software\Microsoft\Windows\CurrentVersion\Run"

# contains the uid that is used by the msi infrastructure to know if the old
# beta is indeed present in the system, can be found in
# lp:ubuntuone-windows-installed/beta/install/UbuntuOne7.wxs
OLD_BETA_UID_KEY = '{13b7ee30-849c-11df-8395-0800200c9a66}'
PROPERTY_BUFFER_SIZE = 256
# error code used by ctypes.windll.msi
ERROR_MORE_DATA = 234
ERROR_SUCCESS = 0
ERROR_UNKNOWN_PRODUCT = 1605
# the name of the properties
VERSION_STRING_PROPERTY = u'VersionString'


class MsiException(Exception):
    """Raised when there are msi issues."""


def uninstall_application():
    """Uninstall Ubuntu One."""
    if hasattr(sys, "frozen"):
        exec_path = os.path.abspath(sys.executable)
    else:
        exec_path = os.path.dirname(__file__)
    folder = os.path.dirname(exec_path)
    uninstall_path = os.path.join(os.path.dirname(folder), "uninstall.exe")
    if os.path.exists(uninstall_path):
        win32api.ShellExecute(None, 'runas',
            uninstall_path,
            '--mode win32', '', 0)


def _get_update_path():
    """Return the path in which the autoupdate command is found."""
    if hasattr(sys, "frozen"):
        exec_path = os.path.abspath(sys.executable)
    else:
        exec_path = os.path.dirname(__file__)
    folder = os.path.dirname(exec_path)
    update_path = os.path.join(folder, "autoupdate-windows.exe")
    if os.path.exists(update_path):
        return update_path
    return None


@defer.inlineCallbacks
def are_updates_present(logger):
    """Return if there are updates for Ubuntu One."""
    update_path = _get_update_path()
    logger.debug('Update path %s', update_path)
    if update_path is not None:
        # If there is an update present we will get 0 and other number
        # otherwise
        retcode = yield getProcessValue(update_path, args=('--mode',
            'unattended'), path=os.path.dirname(update_path))
        logger.debug('Return code %s', retcode)
        if retcode == 0:
            logger.debug('Returning True')
            defer.returnValue(True)
    logger.debug('Returning False')
    defer.returnValue(False)


def perform_update():
    """Spawn the autoupdate process and call the stop funtion."""
    update_path = _get_update_path()
    if update_path is not None:
        # lets call the updater with the commands that are required,
        win32api.ShellExecute(None, 'runas',
            update_path,
            '--unattendedmodeui none', '', 0)


def start_control_panel(with_icon=True):
    """Start the control panel."""
    # If we are in windows and "frozen", assume it's in
    # the same folder as this .exe
    if hasattr(sys, "frozen"):
        cp_dir = os.path.dirname(os.path.abspath(sys.executable))
        cp_path = os.path.join(cp_dir, "ubuntuone-control-panel-qt.exe")
    else:
        cp_path = "ubuntuone-control-panel-qt"
    if with_icon:
        subprocess.Popen([cp_path, "--with-icon"])
    else:
        subprocess.Popen([cp_path])


def add_syncdaemon_to_autostart():
    """Add syncdaemon to the session's autostart."""
    if hasattr(sys, "frozen"):
        sd_path = os.path.join(os.path.dirname(
            os.path.abspath(sys.executable)),
            "ubuntuone-syncdaemon.exe")
        u1cp_path = os.path.join(os.path.dirname(
            os.path.abspath(sys.executable)),
            "ubuntuone-control-panel-qt.exe")
    else:
        return
    registry = _winreg.ConnectRegistry(None, _winreg.HKEY_CURRENT_USER)
    key = _winreg.OpenKey(registry,
        AUTORUN_KEY,
        0, _winreg.KEY_ALL_ACCESS)
    # pylint: disable=E0602
    _winreg.SetValueEx(key, "Ubuntu One", 0, _winreg.REG_SZ, sd_path)
    _winreg.SetValueEx(key, "Ubuntu One Icon", 0, _winreg.REG_SZ,
        u1cp_path + " --minimized")
    _winreg.CloseKey(key)


def default_folders():
    """Return a list of the folders to add by default."""
    # XXXX to be replaced by calls to xdg_base_directory's
    # special_folders
    dll = ctypes.windll.shell32
    buf = ctypes.create_string_buffer(300)
    dll.SHGetSpecialFolderPathA(None, buf, 5, False)
    docs = buf.value
    dll.SHGetSpecialFolderPathA(None, buf, 13, False)
    music = buf.value
    dll.SHGetSpecialFolderPathA(None, buf, 39, False)
    pictures = buf.value
    return [docs, music, pictures]


def is_old_beta_installed():
    """Return whether the old beta is installed in the system."""
    # we try to get the VersisonString for the uid, if we get an error it means
    # that the product is not installed in the system.
    buf_size = 256
    uid_buffer = ctypes.create_unicode_buffer(OLD_BETA_UID_KEY)
    property_buffer = ctypes.create_unicode_buffer(buf_size)
    size = wintypes.DWORD(buf_size)
    result = ctypes.windll.msi.MsiGetProductInfoW(uid_buffer,
                                                  VERSION_STRING_PROPERTY,
                                                  property_buffer,
                                                  ctypes.byref(size))
    # The logic of the following statement is the following. The above code
    # queried the msi system for a property regarding the old beta uid. This
    # uid is the code that uniquely identifies an app in the system. When
    # working with COM properties there are different errors to take into
    # account:
    # ERROR_SUCCESS: We managed to get the data from the property.
    # ERROR_MORE_DATA: The property does exist but the buffer allocated for the
    #                  result is too small.
    # ERROR_UNKNOWN_PROPERTY: The property does not exist.
    # To the above errors we need to add ERROR_UNKNOWN_PRODUCT which is
    # particular to msi and states that the product does not exist.
    # 
    # knowing the above we can deduce the existence of the application by
    # checking if the ERROR_SUCCESS or ERROR_MORE_DATA were returned. This
    # is because the fact that those errors are given state that the uid of the
    # msi is present and that we can get the info.
    return result in (ERROR_SUCCESS, ERROR_MORE_DATA)


def get_property_for_product(product, property_name, buf_size=PROPERTY_BUFFER_SIZE):
    """Retruns the value of a fiven property from a product."""
    property_buffer = ctypes.create_unicode_buffer(buf_size)
    size = wintypes.DWORD(buf_size)
    result = ctypes.windll.msi.MsiGetProductInfoW(product, property_name, property_buffer,
                                                  ctypes.byref(size))
    if result == ERROR_MORE_DATA:
        return get_property_for_product(product, property_name,
                2 * buf_size)
    elif result == ERROR_SUCCESS:
        return property_buffer.value
    else:
        return None


@defer.inlineCallbacks
def uninstall_old_beta():
    """Will remove the old beta from the users machine."""
    # we use the msi lib to be able to uninstall apps
    property_name = u'LocalPackage'
    uninstall_path = get_property_for_product(OLD_BETA_UID_KEY, property_name)
    if uninstall_path is not None:
        # lets remove the package.
        command_line = u'REMOVE=ALL'
        result = yield deferToThread(ctypes.windll.msi.MsiInstallProductW,
                                     uninstall_path, command_line)
        if result != ERROR_SUCCESS:
            raise MsiException('Could not remove old beta.')
    else:
        # the local file is missing so we cannot un install it :(
        raise MsiException('Could not remove old beta.')
